/**
 * JOnAS: Java(TM) Open Application Server
 * Copyright (C) 2010-2011 Bull S.A.S.
 * Contact: jonas-team@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: Builder.java 20932 2011-02-18 17:52:31Z benoitf $
 * --------------------------------------------------------------------------
 */
package org.ow2.jonas.autostart.builder;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.apache.tools.ant.Project;
import org.apache.tools.ant.taskdefs.Copy;
import org.apache.tools.ant.taskdefs.Get;
import org.apache.tools.ant.taskdefs.Move;
import org.apache.tools.ant.taskdefs.Zip;
import org.ow2.jonas.autostart.configuration.Configuration;
import org.ow2.jonas.autostart.configurator.Configurator;
import org.ow2.jonas.autostart.proxy.AutoStartProxy;
import org.ow2.jonas.autostart.unzip.Unzip;
import org.ow2.jonas.autostart.unzip.UnzipException;
import org.ow2.jonas.autostart.utility.Output;
import org.ow2.jonas.autostart.utility.Utility;
import org.ow2.jonas.autostart.version.Version;

/**
 * Create a JOnAS JAR file 'jonas-starter'.
 * @author skabore
 */
public class Builder {

    /**
     * The logger for traces.
     */
    private static final Logger logger = Logger.getLogger(Builder.class.getName());

    /**
     * JOnAS folders location (JOnAS Root & JOnAS Base).
     */
    private List<String> jonasLocation = null;

    /**
     * The jonas-starter location.
     */
    private File starterDefaultLocation = null;

    /**
     * The url where JOnAS zip can be downloaded.
     */
    private String url = null;

    /**
     * The JEE application to deploy location.
     */
    private String[] applicationLocation = null;

    /**
     * The temporary working directory.
     */
    private File workdirectory = null;

    /**
     * The JOnAS Root folder.
     */
    private static final String JONAS_ROOT_FOLDER = "jonasRoot";

    /**
     * The JOnAS Base folder.
     */
    private static final String JONAS_BASE_FOLDER = "jonasBase";

    /**
     * The jonas root zip file.
     */
    private static final String JONAS_ROOT_ZIP_FILE = "jonasRoot.zip";

    /**
     * The jonas base zip file.
     */
    private static final String JONAS_BASE_ZIP_FILE = "jonasBase.zip";

    /**
     * The jonas starter folder.
     */
    private static final String JONAS_STARTER_FOLDER = "jonas-starter";

    /**
     * The jonas starter identifier.
     */
    private String jonasStarterId;

    /**
     * The temporary starter jar file name.
     */
    private String jonasStarterJarFileTmpName;

    /**
     * The JONAS_ROOT path.
     */
    private File jonasRoot;

    /**
     * The JONAS_BASE path.
     */
    private File jonasBase;

    /**
     * The jonas starter jar file name.
     */
    private String starterJarFileName;

    /**
     * The jonas starter jar file location.
     */
    private File starterJarFileLocation;

    /**
     * The file separator.
     */
    private static final String FILE_SEPARATOR = System.getProperty("file.separator");

    /**
     * The temporary working directory name.
     */
    private String tmpWorkDirectoryName;

    /**
     * The output stream.
     */
    private Output output;

    /**
     * The verboseMode.
     */
    private boolean verboseMode;

    /**
     * The createNewJonasBase.
     */
    private boolean createNewJonasBase;

    /**
     * The configuration.
     */
    private Configuration configuration;

    /**
     * The proxy.
     */
    private AutoStartProxy proxy;

    /**
     * The includeJonasClient value.
     */
    private boolean includeJonasClient;

    /**
     * The useJonasFullStarter value.
     */
    private boolean useJonasFullStarter;

    /**
     * SNAPSHOT Repository.
     */
    public static final String MAVEN_SNAPSHOT_REPOSITORY = "http://maven.ow2.org/maven2-snapshot";

    /**
     * Central Repository.
     */
    public static final String MAVEN_CENTRAL_REPOSITORY = "http://repo1.maven.org/maven2";

    /**
     * Build a new Builder with default output stream.
     */
    public Builder() {
        this.output = new Output(logger);
        this.jonasLocation = new ArrayList<String>();
        this.verboseMode = false;
        this.createNewJonasBase = false;
        Date date = new Date();
        DateFormat dateFormat = new SimpleDateFormat("yyyyMMddHHmmss");
        this.jonasStarterId = dateFormat.format(date).toString();
        this.jonasStarterJarFileTmpName = "jonas-starter.jar";
        this.starterJarFileName = "jonas-starter" + this.jonasStarterId + ".jar";
        this.starterJarFileLocation = new File(System.getProperty("user.dir"));
        this.configuration = new Configuration();
        this.proxy = new AutoStartProxy();
        this.includeJonasClient = false;
        this.useJonasFullStarter = false;
    }

    /**
     * Build a new Builder.
     * @param output the output stream.
     */
    public Builder(final Output output) {
        new Builder();
        this.output = output;
    }

    /**
     * Create a temporary working directory.
     */
    public void createTmpWorkDirectory() {
        if (this.workdirectory == null) {
            this.tmpWorkDirectoryName = "builderWD" + this.jonasStarterId;
            this.workdirectory = new File(Utility.getDefaultLocation(), this.tmpWorkDirectoryName);
            Utility.createDirectory(this.workdirectory);
        }
    }

    /**
     * Create JOnAS (Root + Base) zip file : 1 - Download zip file from website
     * or local disk or copy JOnAS folder from local disk. 2 - Unpack JOnAS zip
     * file 3 - Set configuration 4 - Inject application in JOnAS 5 - Zip JOnAS
     * folder.
     */
    public void createJonas() throws BuilderException {
        boolean formExistingFolder = false;
        this.output.write("Getting JOnAS server... Please wait !");
        Project project = new Project();
        if (getJonasRoot() != null) {
            formExistingFolder = true;
            // Zip JOnAS_ROOT folder
            Zip zip = new Zip();
            zip.setBasedir(getJonasRoot());
            zip.setDestFile(new File(this.workdirectory, JONAS_ROOT_ZIP_FILE));
            zip.setUpdate(true);
            zip.setProject(project);
            try {
                zip.execute();
                addJonasLocation(this.workdirectory.getAbsolutePath() + FILE_SEPARATOR + JONAS_ROOT_ZIP_FILE);
            } catch (Exception e) {
                this.output.write(e.getMessage());
            }

            if (getJonasBase() != null) {
                zip.setBasedir(getJonasBase());
                zip.setDestFile(new File(this.workdirectory, JONAS_BASE_ZIP_FILE));
                zip.setUpdate(true);
                zip.setProject(project);
                try {
                    zip.execute();
                    addJonasLocation(this.workdirectory.getAbsolutePath() + FILE_SEPARATOR + JONAS_BASE_ZIP_FILE);
                } catch (Exception e) {
                    this.output.write(e.getMessage());
                }
            }
        } else {
            formExistingFolder = false;
            if (this.url.startsWith("http") || this.url.startsWith("https") || this.url.startsWith("ftp")) {
                // Download JOnAS through URL.
                Get get = new Get();
                try {
                    get.setSrc(new URL(this.url));
                } catch (MalformedURLException ex) {
                    this.output.write(ex.getMessage());
                }
                String file = this.url.substring(this.url.lastIndexOf("/") + 1);
                get.setDest(new File(this.workdirectory, file));
                get.setProject(project);
                get.execute();
                this.jonasLocation.add(0, this.workdirectory.getAbsolutePath() + FILE_SEPARATOR + file);
            } else {
                project = new Project();
                Copy cp = new Copy();
                cp.setProject(project);
                cp.setVerbose(this.verboseMode);
                cp.setFile(new File(this.url));
                cp.setTodir(this.workdirectory);
                cp.execute();
                String file = this.url.substring(this.url.lastIndexOf(FILE_SEPARATOR) + 1);
                this.jonasLocation.add(0, this.workdirectory.getAbsolutePath() + FILE_SEPARATOR + file);
            }
        }
        unzipJonasZipFile(formExistingFolder);
        // Apply configuration.
        if (this.jonasBase == null && this.createNewJonasBase && Utility.isFullJonasFolder(this.jonasRoot)) {
            this.output.write("Advanced configuration task...");
            Utility.createDirectory(new File(this.workdirectory, JONAS_BASE_FOLDER));
            this.jonasBase = new File(this.workdirectory, JONAS_BASE_FOLDER);
            configureJonasServer();
            addJonasLocation(this.jonasBase.getAbsolutePath());
            this.includeJonasClient = true;
        }
        if (!this.includeJonasClient) {
            removeJonasClient();
        }
        this.output.write("Deploying task...");
        injectAppliIntoJonas();
        this.output.write("Packaging task...");
        packJonas();
        this.output.write("Injection task...");
    }

    /**
     * Configures JOnAS server.
     */
    public void configureJonasServer() {
        Configurator configurator = new Configurator(this.jonasRoot.getAbsolutePath(), this.jonasBase.getAbsolutePath(),
                this.configuration, this.workdirectory);
        configurator.doConfiguration();
    }

    /**
     * Copies JOnAS folder(s) into starter folder.
     */
    public void injectJonasIntoStarter() {
        Move mv = new Move();
        mv.setProject(new Project());
        mv.setVerbose(this.verboseMode);
        for (String file : this.jonasLocation) {
            mv.setFile(new File(file));
            mv.setTodir(new File(this.workdirectory.getAbsolutePath() + FILE_SEPARATOR + JONAS_STARTER_FOLDER));
            mv.execute();
        }
    }

    /**
     * Copies application into JOnAS deploy folder.
     */
    public void injectAppliIntoJonas() {
        String destFolder = this.jonasLocation.get(0) + FILE_SEPARATOR;
        if (getJonasBase() != null) {
            destFolder = this.jonasLocation.get(1) + FILE_SEPARATOR;
        }
        destFolder = destFolder + "deploy";
        Get get = new Get();
        Project project = new Project();
        Copy cp = new Copy();
        cp.setProject(new Project());
        cp.setVerbose(this.verboseMode);
        if (this.applicationLocation != null) {
            for (int i = 0; i < this.applicationLocation.length; i++) {
                if (this.applicationLocation[i].startsWith("http://") || this.applicationLocation[i].startsWith("ftp://")) {
                    try {
                        if (Utility.testConnexionURL(new URL(this.applicationLocation[i]))) {
                            try {
                                get.setSrc(new URL(this.applicationLocation[i]));
                                get.setDest(new File(destFolder));
                                get.setProject(project);
                                get.execute();
                            } catch (MalformedURLException ex) {
                                this.output.write(ex.getMessage());
                            }
                        } else {
                            this.output.write("Can't deploy " + this.applicationLocation[i]);
                        }
                    } catch (MalformedURLException ex) {
                        Logger.getLogger(Builder.class.getName()).log(Level.SEVERE, null, ex);
                    }
                } else if (Utility.checkApplication(this.applicationLocation[i])) {
                    cp.setFile(new File(this.applicationLocation[i]));
                    cp.setTodir(new File(destFolder));
                    cp.execute();
                } else {
                    this.output.write("Can't deploy " + this.applicationLocation[i]);
                }
            }
        }
    }

    /**
     * Extracts jonas-starter from jonas-builder archive.
     * @param builderJar the jonas-builder location
     */
    public void initStarter(final String builderJar) throws BuilderException {

        Get get = new Get();
        Project project = new Project();

        try {
            get.setSrc(new URL(builderJar));
            get.setDest(this.workdirectory);
            get.setProject(project);
            get.execute();
            setStarterDefaultLocation(new File(this.workdirectory.getAbsolutePath(), this.jonasStarterJarFileTmpName));
        } catch (MalformedURLException ex) {
            Logger.getLogger(Builder.class.getName()).log(Level.SEVERE, null, ex);
        }

        Unzip unzip = new Unzip(this.output);
        unzip.setVerboseMode(this.verboseMode);
        unzip.setSrc(new File(builderJar));
        setStarterDefaultLocation(new File(this.workdirectory.getAbsolutePath(), this.jonasStarterJarFileTmpName));
        unzip.setDest(this.workdirectory);
        try {
            unzip.execute();
        } catch (UnzipException e) {
            throw new BuilderException("Unable to initialize starter", e);
        }
    }

    /**
     * Set JOnAS Starter jar file location.
     */
    public void setStarter() throws BuilderException {

        Get get = new Get();
        Project project = new Project();

        // Check version
        StringBuilder stringBuilder = new StringBuilder();
        String version = Version.getNumber();

        if (version.contains("SNAPSHOT")) {
            stringBuilder.append(MAVEN_SNAPSHOT_REPOSITORY);
        } else {
            stringBuilder.append(MAVEN_CENTRAL_REPOSITORY);
        }
        stringBuilder.append("/org/ow2/jonas/autostart/");

        if (this.useJonasFullStarter) {
            stringBuilder.append("jonas-full-starter/");
            stringBuilder.append(version);
            stringBuilder.append("/jonas-full-starter-");
            stringBuilder.append(version);
            stringBuilder.append("-full-starter.jar");
        } else {
            stringBuilder.append("jonas-starter/");
            stringBuilder.append(version);
            stringBuilder.append("/jonas-starter-");
            stringBuilder.append(version);
            stringBuilder.append("-starter.jar");

        }

        String starterUrlString = stringBuilder.toString();

        URL starterUrl;
        try {
            starterUrl = new URL(starterUrlString);
        } catch (MalformedURLException e) {
            throw new IllegalStateException("Unable to compute URL from '" + starterUrlString + "'.", e);
        }

        if (Utility.testConnexionURL(starterUrl)) {
            get.setSrc(starterUrl);
            get.setDest(this.workdirectory);
            get.setProject(project);
            get.execute();
            this.jonasStarterJarFileTmpName = starterUrlString.substring(starterUrlString.lastIndexOf("/"), starterUrlString
                    .length());
            setStarterDefaultLocation(new File(this.workdirectory.getAbsolutePath(), this.jonasStarterJarFileTmpName));
        } else {
            throw new BuilderException("Unable to download starter from URL '" + starterUrl + "' : Missing File.");
        }

        Unzip unzip = new Unzip(this.output);
        unzip.setVerboseMode(this.verboseMode);
        unzip.setSrc(getStarterDefaultLocation());
        unzip.setDest(this.workdirectory);
        try {
            unzip.execute();
        } catch (UnzipException e) {
            throw new BuilderException("Unable to set starter", e);
        }
    }

    /**
     * Unzip jonas-starter archive.
     */
    public void unzipStarter() throws BuilderException {
        Unzip unzip = new Unzip(this.output);
        unzip.setVerboseMode(this.verboseMode);
        unzip.setSrc(getStarterDefaultLocation());
        String starterPath = this.workdirectory.getAbsolutePath() + FILE_SEPARATOR + JONAS_STARTER_FOLDER;
        new File(starterPath).mkdir();
        setStarterDefaultLocation(new File(starterPath));
        unzip.setDest(new File(starterPath));
        try {
            unzip.execute();
        } catch (UnzipException e) {
            throw new BuilderException("Unable to unzip starter", e);
        }
    }

    /**
     * Zip jonas-starter folder to jonas-starter.jar.
     */
    public void zipStarter() {
        Project p = new Project();
        Zip zip = new Zip();
        zip.setBasedir(getStarterDefaultLocation());
        zip.setDestFile(new File(this.starterJarFileLocation.getAbsolutePath(), this.starterJarFileName));
        zip.setUpdate(true);
        zip.setProject(p);
        try {
            zip.execute();
            setStarterDefaultLocation(new File(this.starterJarFileLocation.getAbsolutePath(), this.starterJarFileName));
            renameStarterJarFile();
            this.output.write(this.starterJarFileName + " successfully created !");
            this.output.write("Location: " + this.starterDefaultLocation);
        } catch (Exception e) {
            this.output.write("Error in packaging jonas-starter : " + e.getMessage());
        }
    }

    /**
     * Rename and move temporary jar file to starter jar file location.
     */
    private void renameStarterJarFile() {
        File source = this.starterDefaultLocation;
        if (!this.starterJarFileName.endsWith(".jar")) {
            this.starterJarFileName += ".jar";
        }

        File destination = new File(this.starterJarFileLocation.getAbsolutePath(), this.starterJarFileName);
        source.renameTo(destination);
        this.starterDefaultLocation = new File(this.starterJarFileLocation.getAbsolutePath(), this.starterJarFileName);
    }

    /**
     * Unzip JOnAS zip file.
     */
    public void unzipJonasZipFile(final boolean formExistingFolder) throws BuilderException {
        if (getJonasLocation().size() > 0) {
            String rootArchive = getJonasLocation().get(0).substring(getJonasLocation().get(0).lastIndexOf(FILE_SEPARATOR) + 1);
            int start = -1;
            start = rootArchive.indexOf(".zip");
            if (start != -1) {
                Unzip unzip = new Unzip(this.output);
                unzip.setVerboseMode(this.verboseMode);
                unzip.setSrc(new File(getJonasLocation().get(0)));
                File desLocation = this.workdirectory;
                if (formExistingFolder) {
                    Utility.createDirectory(new File(this.workdirectory.getAbsolutePath(), JONAS_ROOT_FOLDER));
                    desLocation = new File(this.workdirectory.getAbsolutePath(), JONAS_ROOT_FOLDER);
                }
                unzip.setDest(desLocation);
                try {
                    unzip.execute();
                } catch (UnzipException e) {
                    throw new BuilderException("Unable to unzip JOnAS archive", e);
                }
            }
            if (start != -1) {
                int start2 = rootArchive.indexOf("-bin");
                if (start2 != -1) {
                    this.jonasLocation.set(0, this.workdirectory.getAbsolutePath() + FILE_SEPARATOR
                            + rootArchive.substring(0, start2));
                } else {
                    this.jonasLocation.set(0, this.workdirectory.getAbsolutePath() + FILE_SEPARATOR
                            + rootArchive.substring(0, start));
                }
            }

            /**
             * Rename JOnAS folder to jonasRoot.
             */
            File source = new File(this.jonasLocation.get(0));
            File destination = new File(this.workdirectory.getAbsolutePath() + FILE_SEPARATOR + JONAS_ROOT_FOLDER);
            this.jonasLocation.set(0, this.workdirectory.getAbsolutePath() + FILE_SEPARATOR + JONAS_ROOT_FOLDER);
            source.renameTo(destination);
            this.jonasRoot = new File(this.jonasLocation.get(0));

            // Unzip jonas base zip file ?
            if (getJonasLocation().size() > 1) {
                String baseArchive = getJonasLocation().get(1).substring(
                        getJonasLocation().get(1).lastIndexOf(FILE_SEPARATOR) + 1);
                int startBase = -1;
                startBase = baseArchive.indexOf(".zip");
                if (startBase != -1) {
                    Unzip unzip = new Unzip(this.output);
                    unzip.setVerboseMode(this.verboseMode);
                    unzip.setSrc(new File(getJonasLocation().get(1)));
                    File desLocation = this.workdirectory;
                    if (formExistingFolder) {
                        Utility.createDirectory(new File(this.workdirectory.getAbsolutePath(), JONAS_BASE_FOLDER));
                        desLocation = new File(this.workdirectory.getAbsolutePath(), JONAS_BASE_FOLDER);
                    }
                    unzip.setDest(desLocation);
                    // unzip.setDest(workdirectory);
                    try {
                        unzip.execute();
                    } catch (UnzipException e) {
                        throw new BuilderException("Unable to unzip JOnAS archive", e);
                    }
                }
                if (startBase != -1) {
                    int startBase2 = baseArchive.indexOf("-bin");
                    if (startBase2 != -1) {
                        this.jonasLocation.set(1, this.workdirectory.getAbsolutePath() + FILE_SEPARATOR
                                + baseArchive.substring(0, startBase2));
                    } else {
                        this.jonasLocation.set(1, this.workdirectory.getAbsolutePath() + FILE_SEPARATOR
                                + baseArchive.substring(0, startBase));
                    }
                }

                /**
                 * Rename JOnAS folder to jonasBase.
                 */
                File sourceBase = new File(this.jonasLocation.get(1));
                File destinationBase = new File(this.workdirectory.getAbsolutePath() + FILE_SEPARATOR + JONAS_BASE_FOLDER);
                this.jonasLocation.set(1, this.workdirectory.getAbsolutePath() + FILE_SEPARATOR + JONAS_BASE_FOLDER);
                sourceBase.renameTo(destinationBase);
                this.jonasBase = new File(this.jonasLocation.get(1));
            }
        }
    }

    /**
     * Zip jonas folder to JonasZip.zip.
     */
    public void packJonas() {
        Project p = new Project();
        Zip zip = new Zip();
        // Zip JOnAS Root folder
        zip.setBasedir(new File(this.workdirectory.getAbsolutePath() + FILE_SEPARATOR + JONAS_ROOT_FOLDER));
        zip.setDestFile(new File(this.workdirectory.getAbsolutePath() + FILE_SEPARATOR + JONAS_ROOT_ZIP_FILE));
        zip.setUpdate(true);
        zip.setProject(p);
        try {
            zip.execute();
            this.jonasLocation.set(0, this.workdirectory.getAbsolutePath() + FILE_SEPARATOR + JONAS_ROOT_ZIP_FILE);
        } catch (Exception e) {
            this.output.write(e.getMessage());
        }

        if (this.jonasBase != null && this.jonasBase.isDirectory()) {
            // Zip JOnAS Base folder
            zip.setBasedir(new File(this.workdirectory.getAbsolutePath() + FILE_SEPARATOR + JONAS_BASE_FOLDER));
            zip.setDestFile(new File(this.workdirectory.getAbsolutePath() + FILE_SEPARATOR + JONAS_BASE_ZIP_FILE));
            zip.setUpdate(true);
            zip.setProject(p);
            try {
                zip.execute();
                this.jonasLocation.set(1, this.workdirectory.getAbsolutePath() + FILE_SEPARATOR + JONAS_BASE_ZIP_FILE);
            } catch (Exception e) {
                this.output.write(e.getMessage());
            }
        }
    }

    /**
     * Check if application location is valid.
     * @param application the application path to check.
     * @return true if the application path is a valid, false otherwise.
     */
    public boolean validApplication(final String application) {
        String[] appli = Utility.parseStrToArray(application, ",");
        for (int i = 0; i < appli.length; i++) {
            /**
             * Where to get application.
             */
            if (appli[i].startsWith("http://") || appli[i].startsWith("ftp://")) {
                try {
                    if (!Utility.testConnexionURL(new URL(appli[i]))) {
                        // output.write("Can't get " + appli[i]);
                        return false;
                    }
                } catch (MalformedURLException ex) {
                    Logger.getLogger(Builder.class.getName()).log(Level.SEVERE, null, ex);
                }
            } else if (!Utility.checkApplication(appli[i])) {
                // output.write("Can't get " + appli[i]);
                return false;
            }
        }
        return true;
    }

    /**
     * @return configuration
     */
    public Configuration getConfiguration() {
        return this.configuration;
    }

    /**
     * Set the configuration.
     * @param configuration the configuration.
     */
    public void setConfiguration(final Configuration configuration) {
        this.configuration = configuration;
    }

    /**
     * @return the verboseMode.
     */
    public boolean getVerboseMode() {
        return this.verboseMode;
    }

    /**
     * Set the verboseMode.
     * @param verboseMode the verboseMode.
     */
    public void setVerboseMode(final boolean verboseMode) {
        this.verboseMode = verboseMode;
    }

    /**
     * @return createNewJonasBase value. If true, a new JOnAS Base is created.
     */
    public boolean getCreateNewJonasBase() {
        return this.createNewJonasBase;
    }

    /**
     * Set the createNewJonasBase value.
     * @param createNewJonasBase the new JOnAS Base value
     */
    public void setCreateNewJonasBase(final boolean createNewJonasBase) {
        this.createNewJonasBase = createNewJonasBase;
    }

    /**
     * @return the output.
     */
    public Output getOutput() {
        return this.output;
    }

    /**
     * Set the output.
     * @param output the output stream.
     */
    public void setOutput(final Output output) {
        this.output = output;
    }

    /**
     * @return JOnAS locations.
     */
    public List<String> getJonasLocation() {
        return this.jonasLocation;
    }

    /**
     * This method modifies jonas location.
     * @param jonasLocation the jonas temporary location
     */
    public void setJonasLocation(final List<String> jonasLocation) {
        this.jonasLocation = jonasLocation;
    }

    /**
     * @return url.
     */
    public String getUrl() {
        return this.url;
    }

    /**
     * This method modifies url.
     * @param url the url where JOnAS can be downloaded
     */
    public void setUrl(final String url) {
        this.url = url;
    }

    /**
     * @return jonas-starter default location.
     */
    public File getStarterDefaultLocation() {
        return this.starterDefaultLocation;
    }

    /**
     * Set jonas-starter default location.
     * @param starterLocation the starter location
     */
    public void setStarterDefaultLocation(final File starterDefaultLocation) {
        this.starterDefaultLocation = starterDefaultLocation;
    }

    /**
     * @return application location.
     */
    public String[] getApplicationLocation() {
        return this.applicationLocation;
    }

    /**
     * This method modifies application location.
     * @param application the application location
     */
    public void setApplicationLocation(final String application) {
        String[] applications = Utility.parseStrToArray(application, ";");
        this.applicationLocation = applications;
    }

    /**
     * @return working directory.
     */
    public File getWorkdirectory() {
        return this.workdirectory;
    }

    /**
     * This method modifies workdirectory.
     * @param workdirectory the working directory
     */
    public void setWorkdirectory(final File workdirectory) {
        this.workdirectory = workdirectory;
    }

    /**
     * @return JOnAS BASE.
     */
    public File getJonasBase() {
        return this.jonasBase;
    }

    /**
     * Modifies JOnAS BASE value.
     * @param jonasBase the JOnAS BASE.
     */
    public void setJonasBase(final File jonasBase) {
        this.jonasBase = jonasBase;
    }

    /**
     * @return JOnAS ROOT.
     */
    public File getJonasRoot() {
        return this.jonasRoot;
    }

    /**
     * Modifies JOnAS ROOT value.
     * @param jonasRoot the JOnAS ROOT.
     */
    public void setJonasRoot(final File jonasRoot) {
        this.jonasRoot = jonasRoot;
    }

    /**
     * @return starterJarFileLocation
     */
    public File getStarterJarFileLocation() {
        return this.starterJarFileLocation;
    }

    /**
     * Set starterJarFileLocation.
     * @param starterJarFileLocation the starterJarFileLocation new value
     */
    public void setStarterJarFileLocation(final File starterJarFileLocation) {
        this.starterJarFileLocation = starterJarFileLocation;
    }

    /**
     * @return starterJarFileName
     */
    public String getStarterJarFileName() {
        return this.starterJarFileName;
    }

    /**
     * Set starterJarFileName.
     * @param starterJarFileName the starterJarFileName new value
     */
    public void setStarterJarFileName(final String starterJarFileName) {
        this.starterJarFileName = starterJarFileName;
    }

    /**
     * @return jonasStarterJarFileTmpName.
     */

    public String getJonasStarterJarFileTmpName() {
        return this.jonasStarterJarFileTmpName;
    }

    /**
     * Set jonasStarterJarFileTmpName value.
     * @param jonasStarterJarFileTmpName the jonasStarterJarFileTmpName new
     *        value.
     */
    public void setJonasStarterJarFileTmpName(final String jonasStarterJarFileTmpName) {
        this.jonasStarterJarFileTmpName = jonasStarterJarFileTmpName;
    }

    /**
     * Add value to JOnAS folderlocation (location 0 means JOnAS ROOT, location
     * 1 means JOnAS BASE folder).
     * @param location the location to add.
     */
    public void addJonasLocation(final String location) {
        this.jonasLocation.add(location);
    }

    /**
     * @returnthe proxy value.
     */
    public AutoStartProxy getAutoStartProxy() {
        return this.proxy;
    }

    /**
     * Set the proxy value.
     * @param proxy the proxy value.
     */
    public void setAutoStartProxy(final AutoStartProxy proxy) {
        this.proxy = proxy;
    }

    /**
     * @return includeJonasClient value.
     */
    public boolean isIncludeJonasClient() {
        return this.includeJonasClient;
    }

    /**
     * Set includeJonasClient value
     * @param includeJonasClient if true $JONAS_ROOT/lib/client.jar will be
     *        included in JOnAS folder.
     */
    public void setIncludeJonasClient(final boolean includeJonasClient) {
        this.includeJonasClient = includeJonasClient;
    }

    /**
     * @return useJonasFullStarter value.
     */
    public boolean isUseJonasFullStarter() {
        return this.useJonasFullStarter;
    }

    /**
     * Set useJonasFullStarter value.
     * @param useJonasFullStarter new value.
     */
    public void setUseJonasFullStarter(final boolean useJonasFullStarter) {
        this.useJonasFullStarter = useJonasFullStarter;
    }

    /**
     * Remove $JONAS_ROOT/lib/client.jar from JOnAS folder.
     */
    private void removeJonasClient() {
        File client = new File(this.jonasRoot.getAbsolutePath(), "lib" + FILE_SEPARATOR + "client.jar");
        if (client.exists()) {
            client.delete();
        }
    }
}
